'''Setup file for python project.'''
import os
import getpass
import requests


def py_init(init, project_path, pname):
    '''
    Setup file creation function.

    Args:
        init: True or False
        project_path: initialize setup.py in project directory

    Returns:
        Initialize successful or not
    '''
    template = '''"""setup file for the python project"""
from setuptools import find_packages, setup

setup(
    version="0.1.0",
    author_email="",
    description="",
    long_description=open("README.md", "r", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    keywords="",
    license="",
    url="",
    package_dir={},
    packages=find_packages(),
    install_requires=[],
    entry_points={},
    extras_require="",
    tests_require="",
    python_requires=">=3.6.0",
    classifiers=[],
'''
    name, user = pname, getpass.getuser()
    inputs = [template, f'    name="{name}",\n', f'    author="{user}"\n)\n']

    os.chdir(project_path)
    open('README.md', 'w').close()
    print("\n📄 Created README.md...")

    gitig = requests.get(
        'https://raw.githubusercontent.com/github/gitignore/master/Python.gitignore')
    with open('.gitignore', 'wb') as gig:
        for chunk in gitig.iter_content():
            gig.write(chunk)
        gig.close()
    print("\n🚀 Fetched .gitignore from GitHub...")

    if init:
        for i in range(len(inputs)):
            with open('setup.py', 'a+') as setup:
                setup.write(inputs[i])
                setup.close()
        print("\n✨ Generated setup.py...")
    else:
        print("No setup.py created...")



'''Testing clifold python project'''
# from clifold.clifold import main
import subprocess
import shutil
import os


python_path = shutil.which('python3')
# command = [f'{python_path} clifold_main.py -g -p -i', f'{python_path} clifold_main.py -ng -np -ni',
#            f'{python_path} clifold_main.py -ng -p -i', f'{python_path} clifold_main.py -ng -np -i',
#            f'{python_path} clifold_main.py -g -np -ni', f'{python_path} clifold_main.py -ng -p -ni',
#            f'{python_path} clifold_main.py -g -np -i', f'{python_path} clifold_main.py -g -p -ni'
#            ]

# for i in range(len(command)):
#     print(f"Testing Test {i}...")
#     subprocess.run([command[i]], check=True)

subprocess.run([python_path, 'clifold/clifold_main.py', 'testing', '-g', '-np', '-i'], check=True)
os.chdir('testing')


def test():
    '''Hard coded testing'''
    assert os.path.exists('src') == True
    assert os.path.exists('tests') == True
    assert os.path.exists('src/testing') == True
    assert os.path.exists('bin') == True
    assert os.path.exists('include') == True
    assert os.path.exists('lib') == True
    assert os.path.exists('README.md') == True
    assert os.path.exists('pyvenv.cfg') == True
    assert os.path.exists('setup.py') == True
    assert os.path.exists('src/testing/__init__.py') == True
    assert os.path.exists('src/testing/main.py') == True
    assert os.path.exists('tests/__init__.py') == True
    assert os.path.exists('tests/test_main.py') == True



"""Main file for clifold."""
import argparse

from clifold.clifold_git import git_init
from clifold.clifold_init import py_init
from clifold.clifold_pkg import pip
from clifold.clifold_project import create


def __version__():
    """return package version"""
    return "0.1.1"


def cli():
    """CLI parsing"""
    parser = argparse.ArgumentParser(
        prog='clif',
        usage='%(prog)s <arg> [options]',
        description='🚀 A CLI tool for scaffolding any Python Projects 🚀',
        conflict_handler='resolve')

    group = parser.add_argument_group('Argument')
    group.add_argument('project_name', help='Project name to create with venv')

    group = parser.add_argument_group('Options')
    group.add_argument('-g', '--git', action='store_true', default=True,
                       dest='git', help='Make git initialization')
    group.add_argument('-ng', '--no-git', action='store_false',
                       dest='git', help='Skip git initialization')

    group.add_argument('-p', '--pkg', action='store_true', default=True,
                       dest='pkg', help='Ask packages to install')
    group.add_argument('-np', '--no-pkg', action='store_false', dest='pkg',
                       help='Skip packages installation')

    group.add_argument('-i', '--init', action='store_true', default=True,
                       dest='init', help='Create setup.py file')
    group.add_argument('-ni', '--no-init', action='store_false',
                       dest='init', help='Skip setup.py file')

    group.add_argument('-V', '--version', action='version',
                       version=__version__(), help='Output version number')
    group.add_argument('-h', '--help', action='help',
                       help='Output usage information')

    args = parser.parse_args()
    pname = args.project_name
    git = args.git
    pkg = args.pkg
    init = args.init

    ppath = create(pname)
    git_init(git, ppath)
    pip(pkg, ppath)
    py_init(init, ppath, pname)


if __name__ == "__main__":
    cli()



'''Install packages with pip.'''
import os
import subprocess
import shutil
import shlex


def pip(pkg, project_path):
    '''
    Packages installation function.

    Args:
        pkg: True or False
        project_path: absolute path to the project

    Returns:
        Packages installed or not installed
    '''
    if pkg:
        pip_path = shutil.which('pip3')
        os.chdir(project_path)
        pkg = input("\n📦 Packages you want to install: ")
        pkgs = shlex.split(pkg)
        args = [pip_path, 'install']
        freeze = [pip_path, 'freeze', '>', 'requirements.txt']
        for i in range(len(args)):
            pkgs.insert(i, args[i])

        print('\n🔰 Downloading packages...\n')
        subprocess.check_call(pkgs)
        subprocess.check_call(freeze)
    else:
        print("\nNo package downloaded...")
